package registry

import (
	"path"

	"github.com/coreos/fleet/etcd"
	"github.com/coreos/fleet/unit"
)

const (
	statePrefix = "/state/"
)

// Get the current UnitState of the provided Job's Unit
func (r *EtcdRegistry) getUnitState(jobName string) *unit.UnitState {
	req := etcd.Get{
		Key:       path.Join(r.keyPrefix, statePrefix, jobName),
		Recursive: true,
	}
	resp, err := r.etcd.Do(&req)

	// Assume the error was KeyNotFound and return an empty data structure
	if err != nil {
		return nil
	}

	var state unit.UnitState
	//TODO: Handle the error generated by unmarshal
	unmarshal(resp.Node.Value, &state)
	return &state
}

// Persist the changes in a provided Machine's Job
func (r *EtcdRegistry) SaveUnitState(jobName string, unitState *unit.UnitState) {
	//TODO: Handle the error generated by marshal
	json, _ := marshal(unitState)

	req := etcd.Set{
		Key:   path.Join(r.keyPrefix, statePrefix, jobName),
		Value: json,
	}
	r.etcd.Do(&req)
}

// Delete the state from the Registry for the given Job's Unit
func (r *EtcdRegistry) RemoveUnitState(jobName string) error {
	req := etcd.Delete{
		Key: path.Join(r.keyPrefix, statePrefix, jobName),
	}
	_, err := r.etcd.Do(&req)
	if isKeyNotFound(err) {
		err = nil
	}
	return err
}
